@@ -13,16 +13,7 @@ from torch import Tensor
 
 
 class SACGP(FixedNoiseGP):
-    """The GP uses Structural Additive Contextual(SAC) kernel.
-
-    Args:
-        train_X: (n x d) X training data.
-        train_Y: (n x 1) Y training data.
-        train_Yvar: (n x 1) Noise variances of each training Y.
-        decomposition: Keys are context names. Values are the indexes of
-            parameters belong to the context. The parameter indexes are in
-            the same order across contexts.
-    """
+    r"""A GP using a Structural Additive Contextual(SAC) kernel."""
 
     def __init__(
         self,
@@ -31,6 +22,15 @@ class SACGP(FixedNoiseGP):
         train_Yvar: Tensor,
         decomposition: Dict[str, List[int]],
     ) -> None:
+        r"""
+        Args:
+            train_X: (n x d) X training data.
+            train_Y: (n x 1) Y training data.
+            train_Yvar: (n x 1) Noise variances of each training Y.
+            decomposition: Keys are context names. Values are the indexes of
+                parameters belong to the context. The parameter indexes are in
+                the same order across contexts.
+        """
         super().__init__(train_X=train_X, train_Y=train_Y, train_Yvar=train_Yvar)
         self.covar_module = SACKernel(
             decomposition=decomposition,
@@ -42,26 +42,10 @@ class SACGP(FixedNoiseGP):
 
 
 class LCEAGP(FixedNoiseGP):
-    r"""The GP with Latent Context Embedding Additive (LCE-A) Kernel.
+    r"""A GP using a Latent Context Embedding Additive (LCE-A) Kernel.
+
     Note that the model does not support batch training. Input training
     data sets should have dim = 2.
-
-    Args:
-        train_X: (n x d) X training data.
-        train_Y: (n x 1) Y training data.
-        train_Yvar: (n x 1) Noise variance of Y.
-        decomposition: Keys are context names. Values are the indexes of
-            parameters belong to the context. The parameter indexes are in the
-            same order across contexts.
-        cat_feature_dict: Keys are context names and values are list of categorical
-            features i.e. {"context_name" : [cat_0, ..., cat_k]}. k equals to number
-            of categorical variables. If None, we use context names in the
-            decomposition as the only categorical feature i.e. k = 1
-        embs_feature_dict: Pre-trained continuous embedding features of each context.
-        embs_dim_list: Embedding dimension for each categorical variable. The length
-            equals to num of categorical features k. If None, emb dim is set to 1
-            for each categorical variable.
-        context_weight_dict: Known population Weights of each context.
     """
 
     def __init__(
@@ -76,6 +60,25 @@ class LCEAGP(FixedNoiseGP):
         embs_dim_list: Optional[List[int]] = None,
         context_weight_dict: Optional[Dict] = None,
     ) -> None:
+        r"""
+        Args:
+            train_X: (n x d) X training data.
+            train_Y: (n x 1) Y training data.
+            train_Yvar: (n x 1) Noise variance of Y.
+            decomposition: Keys are context names. Values are the indexes of
+                parameters belong to the context. The parameter indexes are in the
+                same order across contexts.
+            cat_feature_dict: Keys are context names and values are list of categorical
+                features i.e. {"context_name" : [cat_0, ..., cat_k]}, where k is the
+                number of categorical variables. If None, we use context names in the
+                decomposition as the only categorical feature, i.e., k = 1.
+            embs_feature_dict: Pre-trained continuous embedding features of each
+                context.
+            embs_dim_list: Embedding dimension for each categorical variable. The length
+                equals the number of categorical features k. If None, the embedding
+                dimension is set to 1 for each categorical variable.
+            context_weight_dict: Known population weights of each context.
+        """
         super().__init__(train_X=train_X, train_Y=train_Y, train_Yvar=train_Yvar)
         self.covar_module = LCEAKernel(
             decomposition=decomposition,
