@@ -127,6 +127,8 @@ def model_list_to_batched(model_list: ModelListGP) -> BatchedMultiOutputGPyTorch
         >>> list_gp = ModelListGP(gp1, gp2)
         >>> batch_gp = model_list_to_batched(list_gp)
     """
+    was_training = model_list.training
+    model_list.train()
     models = model_list.models
     _check_compatibility(models)
 
@@ -150,10 +152,15 @@ def model_list_to_batched(model_list: ModelListGP) -> BatchedMultiOutputGPyTorch
             raise UnsupportedError("All models must have the same fidelity parameters.")
         kwargs.update(init_args)
 
+    # add batched kernel, except if the model type is SingleTaskMultiFidelityGP,
+    # which does not have a `covar_module`
+    if not isinstance(models[0], SingleTaskMultiFidelityGP):
+        batch_length = len(models)
+        covar_module = _batched_kernel(models[0].covar_module, batch_length)
+        kwargs["covar_module"] = covar_module
+
     # construct the batched GP model
     input_transform = getattr(models[0], "input_transform", None)
-    if input_transform is not None:
-        input_transform.train()
     batch_gp = models[0].__class__(input_transform=input_transform, **kwargs)
     adjusted_batch_keys, non_adjusted_batch_keys = _get_adjusted_batch_keys(
         batch_state_dict=batch_gp.state_dict(), input_transform=input_transform
@@ -193,7 +200,47 @@ def model_list_to_batched(model_list: ModelListGP) -> BatchedMultiOutputGPyTorch
     # load the state dict into the new model
     batch_gp.load_state_dict(batch_state_dict)
 
-    return batch_gp
+    return batch_gp.train(mode=was_training)
+
+
+def _batched_kernel(kernel, batch_length: int):
+    """Adds a batch dimension of size `batch_length` to all non-scalar
+    Tensor parameters that govern the kernel function `kernel`.
+    NOTE: prior or constraint parameters are excluded from batching.
+    """
+    # copy just in case there are non-tensor parameters that are passed by reference
+    kernel = deepcopy(kernel)
+    search_str = "raw_outputscale"
+    for key, attr in kernel.state_dict().items():
+        if isinstance(attr, Tensor) and (
+            attr.ndim > 0 or (search_str == key.rpartition(".")[-1])
+        ):
+            attr = attr.unsqueeze(0).expand(batch_length, *attr.shape).clone()
+            set_attribute(kernel, key, torch.nn.Parameter(attr))
+    return kernel
+
+
+# two helper functions for `batched_kernel`
+# like `setattr` and `getattr` for object hierarchies
+def set_attribute(obj, attr: str, val):
+    """Like `setattr` but works with hierarchical attribute specification.
+    E.g. if obj=Zoo(), and attr="tiger.age", set_attribute(obj, attr, 3),
+    would set the Zoo's tiger's age to three.
+    """
+    path_to_leaf, _, attr_name = attr.rpartition(".")
+    leaf = get_attribute(obj, path_to_leaf) if path_to_leaf else obj
+    setattr(leaf, attr_name, val)
+
+
+def get_attribute(obj, attr: str):
+    """Like `getattr` but works with hierarchical attribute specification.
+    E.g. if obj=Zoo(), and attr="tiger.age", get_attribute(obj, attr),
+    would return the Zoo's tiger's age.
+    """
+    attr_names = attr.split(".")
+    while attr_names:
+        obj = getattr(obj, attr_names.pop(0))
+    return obj
 
 
 def batched_to_model_list(batch_model: BatchedMultiOutputGPyTorchModel) -> ModelListGP:
@@ -212,6 +259,8 @@ def batched_to_model_list(batch_model: BatchedMultiOutputGPyTorchModel) -> Model
         >>> batch_gp = SingleTaskGP(train_X, train_Y)
         >>> list_gp = batched_to_model_list(batch_gp)
     """
+    was_training = batch_model.training
+    batch_model.train()
     # TODO: Add support for HeteroskedasticSingleTaskGP.
     if isinstance(batch_model, HeteroskedasticSingleTaskGP):
         raise NotImplementedError(
@@ -222,8 +271,6 @@ def batched_to_model_list(batch_model: BatchedMultiOutputGPyTorchModel) -> Model
             "Conversion of MixedSingleTaskGP is currently not supported."
         )
     input_transform = getattr(batch_model, "input_transform", None)
-    if input_transform is not None:
-        input_transform.train()
     outcome_transform = getattr(batch_model, "outcome_transform", None)
     batch_sd = batch_model.state_dict()
 
@@ -278,7 +325,7 @@ def batched_to_model_list(batch_model: BatchedMultiOutputGPyTorchModel) -> Model
         model.load_state_dict(sd)
         models.append(model)
 
-    return ModelListGP(*models)
+    return ModelListGP(*models).train(mode=was_training)
 
 
 def batched_multi_output_to_single_output(
@@ -315,6 +362,8 @@ def batched_multi_output_to_single_output(
         >>> batch_mo_gp = SingleTaskGP(train_X, train_Y)
         >>> batch_so_gp = batched_multioutput_to_single_output(batch_gp)
     """
+    was_training = batch_mo_model.training
+    batch_mo_model.train()
     # TODO: Add support for HeteroskedasticSingleTaskGP.
     if isinstance(batch_mo_model, HeteroskedasticSingleTaskGP):
         raise NotImplementedError(
@@ -328,8 +377,6 @@ def batched_multi_output_to_single_output(
             "Conversion of models with custom likelihoods is currently unsupported."
         )
     input_transform = getattr(batch_mo_model, "input_transform", None)
-    if input_transform is not None:
-        input_transform.train()
     batch_sd = batch_mo_model.state_dict()
 
     # TODO: add support for outcome transforms.
@@ -352,7 +399,7 @@ def batched_multi_output_to_single_output(
         input_transform=input_transform, **kwargs
     )
     single_outcome_model.load_state_dict(batch_sd)
-    return single_outcome_model
+    return single_outcome_model.train(mode=was_training)
 
 
 def _get_adjusted_batch_keys(
