@@ -4,6 +4,7 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from botorch.models.transforms.factory import get_rounding_input_transform
 from botorch.models.transforms.input import (
     ChainedInputTransform,
     Normalize,
@@ -20,6 +21,7 @@ from botorch.models.transforms.outcome import (
 
 
 __all__ = [
+    "get_rounding_input_transform",
     "Bilog",
     "ChainedInputTransform",
     "ChainedOutcomeTransform",
